require 'open3'
require 'shellwords'
require 'rainbow'
require 'git-blame-color/pager'
require 'git-blame-color/ext/argv'
require 'git-blame-color/ext/array'


module Git
  module BlameColor
    class << self
      def application
        @application || Git::BlameColor::Application.new
      end
    end

    class Application
      include Pager

      def run(argv)
        Rainbow.enabled = true
        die 1, 'git blame-color does not support the --incremental option' if ARGV.incremental?
        stdout, stderr, status = Open3.capture3('git', 'blame', '--line-porcelain', *ARGV)
        die status.exitstatus, stderr unless status.success?
        lines = stdout.split($/)
        stats = nil
        if ARGV.show_stats?
          stats = lines.last(3)
          lines = lines.drop_last(3)
        end
        commits = parse_commits(lines, [])
        author_width = longest_author_name_width(commits)
        line_width = max_line_number_width(commits)
        opts = {
          :show_boundary_commits => !ARGV.blank_boundary_commits?,
          :long_hashes => ARGV.long_hash?,
          :raw_timestamps => ARGV.raw_timestamps?,
        }
        formatter = Git::BlameColor::Formatter.new(author_width, line_width, opts)
        page
        commits.each do |c|
          puts formatter.format(c)
        end
        puts stats.join("\n") if stats
      end

      private

      def die(status, msg)
        STDERR.puts msg
        exit status
      end

      def parse_commits(commits, acc)
        return acc if commits.empty?
        lines = commits.take_while { |line| line =~ /^[^\t]/ }
        lines << commits.find { |line| line =~ /^\t/ }
        commit = Git::BlameColor::Commit.new(lines)
        acc << commit
        commits = commits.drop(lines.count)
        parse_commits(commits, acc)
      end

      def longest_author_name_width(commits)
        commits.longest_string_length_by_key(:author)
      end

      def max_line_number_width(commits)
        commits.longest_string_length_by_key(:line_number)
      end
    end
  end
end
