require 'rainbow'


module Git
  module BlameColor
    class Formatter
      attr_reader :author_name_width, :line_number_width

      def initialize(author_name_width, line_number_width, options={})
        @author_name_width = author_name_width
        @line_number_width = line_number_width
        @options = options
      end

      def long_hashes?
        !!@options[:long_hashes]
      end

      def show_boundary_commits?
        @options.include?(:show_boundary_commits) ? !!@options[:show_boundary_commits] : true
      end

      def raw_timestamps?
        !!@options[:raw_timestamps]
      end

      def format(commit)
        parts = [
          color_coded_hash(commit),
          Rainbow(author_to_s(commit)).blue,
          Rainbow(timestamp_to_s(commit)).green,
          Rainbow(line_number_to_s(commit)).black.bright,
          commit.edited_line
        ]
        parts.join(' ')
      end

      private

      def hash_length
        long_hashes? ? 40 : 8
      end

      def hash_to_s(commit)
        if commit.boundary?
          "^#{commit.hash[0,hash_length-1]}"
        else
          commit.hash[0,hash_length]
        end
      end

      def color_coded_hash(commit)
        if show_boundary_commits?
          Rainbow(hash_to_s(commit)).yellow
        else
          ' ' * 8
        end
      end

      def author_to_s(commit)
        sprintf('%-*s', author_name_width, commit.author)
      end

      def line_number_to_s(commit)
        sprintf('%*s', line_number_width, commit.line_number.to_s)
      end

      def timestamp_to_s(commit)
        if raw_timestamps?
          commit.timestamp
        else
          timestamp_to_time(commit).strftime('%Y-%m-%d')
        end
      end

      def timestamp_to_time(commit)
        stamp = commit.timestamp
        zone = commit.timezone
        front_zone, back_zone = zone[0..-3], zone[-2,2]
        zone = "#{front_zone}:#{back_zone}"
        t = Time.at(stamp.to_i)
        Time.new(t.year, t.month, t.day, t.hour, t.min, t.sec, zone)
      end
    end
  end
end
